//==============================================================================
//
// Title:      rsspecan_clang_measurement_Example
// Purpose:    Demonstration example on how to use rsspecan instrument driver
//             to Initiate a measurement sweep and read a sweep result
// Notes:      To run the example you need to install rsspecan VXI Plug&Play
//             instrument driver version 3.0.0 or later.
//             Default locations of instrument driver files on 32bit Windows:
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\rsspecan\rsspecan.fp
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\include\rsspecan.h
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\lib\msc\rsspecan.lib
//             Default locations of instrument driver files on 64bit Windows:
//             C:\Program Files\IVI Foundation\VISA\Win64\rsspecan\rsspecan.fp
//             C:\Program Files\IVI Foundation\VISA\Win64\include\rsspecan.h
//             C:\Program Files\IVI Foundation\VISA\Win64\lib\msc\rsspecan.lib
//
// Created on: 09/2016 by Miloslav Macko.
// Copyright:  Rohde & Schwarz GmbH & Co. KG. All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include <rsspecan.h>

//==============================================================================
// Constants
#define STRING_LENGTH 512  

//==============================================================================
// Types

//=============x=================================================================
// Static global variables
static ViString  gs_sResourceName = "TCPIP::127.0.0.1::INSTR";
static ViBoolean gs_hIDQuery = VI_TRUE;
static ViBoolean gs_hResetDevice = VI_TRUE;

//==============================================================================
// Global variables
ViSession io = 0;

//==============================================================================
// Global functions
/// HIFN Checks for instrument error
/// HIRET Error code
/// HIPAR status/Status code to be checked, returned from instrument driver function call
ViStatus checkError (ViStatus status)
{
	ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH * 2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        rsspecan_error_message (io, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (io, &error, error_message);
        
        if (error != 0)
        {
            sprintf (p2buf, "Secondary Error: Instrument is reporting the following error (SYST:ERR?): %d, %s\n", error, error_message);
        }
        
        rsspecan_close (io);
    printf ("\nERROR!\n\n");
    printf (error_buffer);
    printf ("\nPress ENTER to continue...");
    (void) getchar();
        
    }
    return status;
}

//==============================================================================
// Macros
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/// HIFN  The main entry-point function.
/// HIRET Returns 0 if successful. Otherwise returns instrument driver error 
/// HIRET code
int main (void)
{

  int i;
  ViInt32  errorCode, points;
  ViReal64 amplitude[1024];
  ViStatus status = VI_SUCCESS;
  ViChar   serror_message [256] = "No Error";
  ViChar   id_response[1024], sErrorMessage[1024];

  
  printf ("Initializing instrument '%s' ... ", gs_sResourceName);
  CHECKERR (rsspecan_init (gs_sResourceName, gs_hIDQuery, gs_hResetDevice, &io));
  printf ("success.\n\n");
  
  printf("IMPORTANT!!! Changing to Single Sweep ... ");
  CHECKERR(rsspecan_ConfigureAcquisition(io, 0, VI_FALSE, 1));
  printf("success.\n\n");
  
  printf ("Disabling remote display update for the fastest performance ... ");
  CHECKERR (rsspecan_ConfigureDisplayUpdate (io, VI_TRUE));
  printf ("success.\n\n");
  
  //printf ("Reading instrument *IDN? string ... ");
  //CHECKERR(rsspecan_IDQueryResponse(io, id_response));
  //printf ("success.\n\n");
  //printf ("Connected instrument: %s\n\n", id_response);

  printf("Configuring the settings ... ");
  CHECKERR(rsspecan_ConfigureFrequencyCenterSpan(io, 0, 3E9, 10E6));
  CHECKERR(rsspecan_ConfigureSweepCoupling(io, 0, RSSPECAN_VAL_COUPLING_RBW, 10E3));
  CHECKERR(rsspecan_ConfigureSweepCoupling(io, 0, RSSPECAN_VAL_COUPLING_VBW, 1E3));
  CHECKERR(rsspecan_error_query(io, &errorCode, sErrorMessage));
  
  printf("Running single acquisition ... ");
  CHECKERR(rsspecan_Initiate(io, 0, 1000));
  printf("success.\n\n");

  //reading the sweep trace points of the trace 1
  printf("Sweep points:\n");
  CHECKERR(rsspecan_FetchYTrace(io, 0, 1, 1024, &points, amplitude));
  for (i = 0; i < points; i++)
  {
	  printf("%0.2f, ", amplitude[i]);
  }

  printf ("Closing the session to instrument '%s' ... ", gs_sResourceName);
  CHECKERR (rsspecan_close (io));
  printf ("success.\n\n");
  
  rsspecan_error_message (0, status, serror_message);

  printf ("\nPress ENTER to continue...");
  (void) getchar();
  
  return status;
}

